#!/bin/perl -w
use strict;
use warnings;

sub help{
	if(grep {$_ eq "-h"} @ARGV){
		print "############################################################################################################################################\n";
		print "	This script can be used to calculate OPSS (Orthology_Protein-coding genes_Support_Score) for each OPPs used in OPPs_identify.sh as Step4. \n";
		print "	Usage: perl Step4.pl OPPs_MAAs-based.pair OPPs_proteins-based.pair BBHs.pair > Final.opps \n";
		print "	Input: (1) OPPs_MAAs-based.pair: the OPPs results obtained by using MAAs as genomic markers as above steps \n";
		print "		(2) OPPs_proteins-based.pair: the OPPs results obtained by using protein-coding genes as genomic markers to perform above all steps \n";
		print "		(3) BBHs.pair: BBH.results were obtained by blastp all_vs_all and extract BBH (Bidirectional Best Hits) relationships with BBH_extract.pl \n";
		print "		(4) p_weight: indicating a [0,1] number as weight of score obtained by protein-based method \n";
		print "	Output: Final.opps \n";
		print "	Final.opps: six columns as as ref_geneID, spec_geneID, M_score (MAAs-based), P_score (proteins-based), BBH_score?(0:no, 2:yes) and OPSS_score (= M_score + P_score + BBH_score) \n";
		print "#############################################################################################################################################\n";
		}
	}

&help();



my ($M, $P, $B, $p_weight)=@ARGV;
my (%M_hash, %P_hash, %B_hash); # key: opps; value: 

&collect_hash($M,$P,$B);

foreach my $i(sort keys%M_hash){
	my $score=0;
	my $m_score=0;
	my $p_score=0;
	my $b_score=0;
	my @mll=split /\|/, $M_hash{$i};
	foreach my $mi(@mll){
		$m_score=$m_score+&line2score($mi);
	}
	my @pll;
	if (exists $P_hash{$i}){
		@pll=split /\|/, $P_hash{$i};
		foreach my $pi(@pll){
			$p_score=$p_score+(&line2score($pi));
		}
	}
$b_score=$B_hash{$i};
$score=$m_score+$p_score*$p_weight+$b_score;
print "$i\t$m_score\t$p_score\t$b_score\t$score\n";
}




sub line2score{
(my $l)=@_;
my $score=0;
my @all=split /\t/, $l;
####ways###
if ($all[2]=~/2way\-/){
	(my $t)=($all[2]=~/2way\-(\d+)/);
	$score=$score+$t;
	}else{
		(my $t)=($all[2]=~/(\d+)way/);
		$score=$score+$t;
		}
return $score;
}



sub collect_hash{
my ($M, $P, $B)=@_;
####collect M_hash from MAAs-based results
open IN, $M;
while(<IN>){
	chomp($_);
	my @all=split /\t/, $_;
	if (!exists $M_hash{$all[0]."\t".$all[1]}){
			$M_hash{$all[0]."\t".$all[1]}=$_;
		}else{
			$M_hash{$all[0]."\t".$all[1]}=$M_hash{$all[0]."\t".$all[1]}."\|".$_;
		}
	}
close IN;

####collect P_hash from proteins-based results
open IN, $P;
while(<IN>){
	chomp($_);
	my @all=split /\t/, $_;
	if (!exists $P_hash{$all[0]."\t".$all[1]}){
			$P_hash{$all[0]."\t".$all[1]}=$_;
		}else{
			$P_hash{$all[0]."\t".$all[1]}=$P_hash{$all[0]."\t".$all[1]}."\|".$_;
		}
	}
close IN;

####collect B_hash from BBH results
open IN, $B;
my $b_score;
while(<IN>){
	chomp($_);
	my @all=split /\t/, $_;
	### BBH relationship could be considered as two species level evidence
	if ($all[2] == 1){
		$b_score=2;
	}else{
		$b_score=0;
	}
	$B_hash{$all[0]."\t".$all[1]}=$b_score;
	}
close IN;
}
