# Author by songht.CMB.bnu 
# Date: 2017-12-01
#!/bin/sh
export perl="/userset/perl"


################################################################################################################################################################################
# Prepare dataset: n-way(2way-n)_segments_gff3/* produced by collinear_detection.sh 	                			
# Description:
#	Step 1  Collect the genesets located in all n-way[2way-n] collinear_segments			   			   	  
#	Step 2  geneset2genemap: For two species (with cucumber as ref), select the genes which located in the same colliner segment.														
#	Step 3  genemap_blast_filter: For each pair of genes located in the same segment, if they share no similarity (blast E-value), they will be filtered out. 						
#	Step 4  For each pair of OPPs, to calculate its collinear segments support score (OPSS: Ortholog_Protein-coding genes_Support_Score) 											
#	Dependency tools: bedtools2-master, pl_scripts/*.pl, BBH.pair(obtained by blastp all_vs_all and extract BBH (Bidirectional Best Hits) relationships )											
# Usage: sh collinear.sh 			   																			
#	Input: n-way[2way-n]_collinear.gff3, spec_pro_gff3 including protein-coding genes annotation    			
#	Output: Final.opps 															                    			
#	output with six columns as ref_geneID, spec_geneID, M_score (MAAs-based), P_score (proteins-based), BBH_score?(0:no, 2:yes) and OPSS_score (= M_score + P_score + BBH_score)'	
##################################################################################################################################################################################


echo 'Step1 Collect the genesets located in all n-way[2way-n] collinear_segments'
echo 'Input: (1) n-way[2way-n]_collinear_segments.gff; (2) protein-coding_genes.gff3'
echo 'Output: n-way[2way-n]_geneset.table with two columns as geneID and multipliconID'
echo 'Step1 start at'
date

for spec in Cucumber Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-ly Arab-th Viti-vi Sola-tu Seta-ti Oryza-brach Brach
	do
		export segments_gff3="/Userset/segments_gff3"  # collinear segments gff3 files 
		export spec=$spec # one of 15 angiosperm plants
		export pro_gff_dir="/Userset/pro_gff_dir" # dir including protein-coding_genes.gff3
		export cover_ratio="0.5" # indicating the cover ratio (0~1) of intersection between one collinear segment and target protein-coding gene with gene's total length as denominator
		export nway="" # the level of collinear: n-way(n=3~15) OR 2way-n(n=2~15)
		export output="/Userset/geneset"
		export bedtools='/Userset/bedtools2-master/bin'
		mkdir tmp # store the tmp files
		# Step 1.1 collect the scaffold in each species which includes the protein-coding genes and saved as pro_scafold.list
		awk '{print $1}' $pro_gff_dir/$spec\_pro.gff3 | sort -u >  $output/pro_scaffold.list

		# Sttep 1.2 collect the collinear segments which locate in scaffolds including genes
		for chr in `cat pro_scaffold.list`; do grep "$spec" $segments_gffs/$nway/n-way[2way-n]\_segments.gff3 | awk -v chr=$chr  '{if ($1 == chr){print $0}}'; done > ./tmp/$nway\_$spec_collinear.gff

		# Step 1.3 Using intersectBed function from bedtoos to obtain the geneset located in one multiplcon segment
		$bedtools/intersectBed -nonamecheck -b ./tmp/$nway\_$spec_collinear.gff -a $pro_gff_dir/$spec\_pro.gff3  -f $cover_ratio -wao  | sort -u | grep "\-1" -v  | awk '{print $9"\t"$18}'| awk -F ";" '{print $1}' > $output/$spec\_$nway\_geneset.table
		rm -fr tmp
	done
echo 'Step1 Finished at'
date


########################################################################################
########################################################################################


echo 'Step2 geneset2genemap: For two species (with cucumber as ref), select the genes which located in the same collinear segment'
echo 'Input: (1) ref_geneset.table;(2) spec_geneset.table'
echo 'Output: n-way[2way-n]_genemap.table with three columns as segmentID, ref_geneID and spec_geneID'
echo 'Step2 start at'
date

export wkdir="/Userset/"
export ref="Cucumber" # Cucumber as reference genome
export Step2="/Userset/pl_sctript/Step2.pl"
export output="/Userset/genemap"
for i in Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-ly Arab-th Viti-vi Sola-tu Seta-ti Oryza-brach Brach
	do
		export spec=$i # one of 15 angiosperm plants
		export nway="" # the level of collinear: n-way(n=3~15) OR 2way-n(n=2~15)
		perl $Step2 $wkdir/${ref}_${nway}_geneset.table $wkdir/${spec}_${nway}_geneset.table > $output/${ref}_${spec}_${nway}_genemap.table
	done

echo 'Step2 Finished at'
date

########################################################################################
########################################################################################


echo 'Step3 genemap_blast_filter (Step3.1) and collect the way infor for each OPPs (Step3.2) '
echo 'Inout: (1) ref_spec_nway_genemap.table; (2) ref_spec.blast'
echo 'Ooutput: n-way[2way-n]_all.opps with 5 columns including ref_geneID, spec_geneID, spec, n-way[2way-n], segmentID'
echo 'Step3 start at'
date


################################################
echo 'Step3.1 For each pair of genes located in the same segment, if they share no similarity (blast E-value), they will be filtered out'
export wkdir="/Userset/"
export ref="Cucumber" # Cucumber as reference genome
export Step3.1="/Userset/pl_script/Step3.1.pl"
export blast_file="/userset/blast_results/"
export blast_evalue_cutoff="10E-5"
export output="/Userset/genemap"
for i in Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-ly Arab-th Viti-vi Sola-tu Seta-ti Oryza-brach Brach
	do
		export spec=$i # one of 15 angiosperm plants
		export nway="" # the level of collinear: n-way(n=3~15) OR 2way-n(n=2~15)
		perl $Step3.1 $wkdir/$ref\_$spec\_$nway\_genemap.table ${blast_file}/$ref\_$spec.blast ${blast_evalue_cutoff} | sort -u > $output/$ref\_$spec\_$nway\_good.genemap
	done

################################################

echo 'Step3.2 collect the nway[2way-n] information for each OPPs'
export Step3.2="/Userset/pl_scripts/Step3.2.pl"
export genemap="/Userset/genemap"
export output="/Userset/OPPs"

#performing the n-way results
for n in {3..15}
	do
		for spec in Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-ly Arab-th Viti-vi Sola-tu Seta-ti Oryza-brach Brach
			do
				perl $Step3.2 $genemap/$ref\_$spec\_${n}-way\_good.genemap $spec ${n}way > $output/$ref\_$spec\_{n}way.opps
			done
	done
cat $output/*n-way.opps > $wkdir/geneset2genemap/OPPs/n-way_all.opps
awk '{print "Cucumber\."$1"\t"$3"."$2"\t"$4"\t"$5}' $output/n-way_all.opps > $output/MAAs-based-nway.opps


#performing the 2way-n results
for n in {2..15}
	do
		for spec in Cucu-me Water-melo Malus-dome Glyc-max Popu-tr Citrus Brass-ra Arab-ly Arab-th Viti-vi Sola-tu Seta-ti Oryza-brach Brach
			do
				perl $Step3.2 $genemap/$ref\_$spec\_2way-${n}\_good.genemap $spec 2way-${n} > $output/$ref\_$spec\_2way-${n}.opps
			done
	done
cat $output/*2way-n.opps > $output/2way-n_all.opps
awk '{print "Cucumber\."$1"\t"$3"."$2"\t"$4"\t"$5}' $output/2way-n_all.opps > $output/MAAs-based-2way-n.opps


echo 'Step3 Finished at'
date


########################################################################################
########################################################################################


echo 'Step4 For each pair of OPPs, to calculate its collinear segments support score (OPSS: Ortholog_Protein-coding genes_Support_Score) '
echo 'Input: (1) OPPs_MAAs-based.pair: the OPPs results obtained by using MAAs as genomic markers as above steps'
echo ' (2) OPPs_proteins-based.pair: the OPPs resuts obtained by using protein-coding genes as genomic markers to perform above all steps'
echo ' (3) BBHs.pair: BBH.results were obtained by blastp all_vs_all and extract BBH (Bidirectional Best Hits) relationships with BBH_extract.pl'
echo ' (4) p_weight: indicating a [0,1] number as weight of score obtained by protein-based method'
echo 'Output: Final.opps with six columns as ref_geneID, spec_geneID, M_score (MAAs-based), P_score (proteins-based), BBH_score?(0:no, 2:yes) and OPSS_score (= M_score + P_score + BBH_score)'
echo 'Step4 start at'
export wkdir="/Userset/OPPs"
export Step5="/Userset/pl_scripts/Step4.pl"
export MAAs-based.pair="/Userset/OPPs/OPPs_MAAs-based.pair"
export Proteins-based.pair="/Userset/OPPs/OPPs_proteins-based.pair"
export BBH="/Userset/blast_results/BBH.results"
export p_weight="0.5"

perl $Step4 $MAAs-based.pair $Priteins-based.pair $BBH $p_weight > $wkdir/Final.opps

echo 'Step4 Finished at'
date






